@@ -1,8 +1,8 @@
 /*=============================================================
- * interp.c -- Interprets program statements
+ * interp.c -- Interpret program statements
  * Copyright(c) 1991-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 16 Aug 93
- *   3.0.0 - 26 Jul 94
+ *   3.0.0 - 26 Jul 94    3.0.2 - 25 Mar 95
  *===========================================================*/
 
 #include "standard.h"
@@ -14,12 +14,19 @@
 
 extern STRING llprograms;
 
-FILE *ifp = NULL, *ofp = NULL;
-TABLE proctab, globtab, functab;
-INT rplineno;
-INT semerrors = 0;
-STRING ierror = "Error: line %d: ";
-static STRING noreport = "No report was generated.";
+STRING Pfname = NULL;	/* file to read program from */
+FILE *Pinfp  = NULL;	/* file to read program from */
+FILE *Poutfp = NULL;	/* file to write program output to */
+STRING Pinstr = NULL;	/* string to read program from */
+STRING Poutstr = NULL;	/* string to write program output to */
+INT Plineno = 1;
+INT Perrors = 0;
+LIST Plist;		/* list of program files still to read */
+
+TABLE filetab, proctab, globtab, functab;
+STRING ierror = (STRING) "Error: file \"%s\": line %d: ";
+static STRING noprogram = (STRING) "No program was run.";
+static STRING qrptname = (STRING) "What is the name of the program?";
 
 /*=====================================
  * initinterp -- Initialize interpreter
@@ -28,7 +35,7 @@ initinterp ()
 {
 	initrassa();
 	initset();
-	semerrors = 0;
+	Perrors = 0;
 }
 /*===================================
  * finishinterp -- Finish interpreter
@@ -37,62 +44,140 @@ finishinterp ()
 {
 	finishrassa();
 }
-/*==========================================
- * interp_report -- Interpreter main routine
- *========================================*/
-interp_report ()
+/*==============================================
+ * interp_program -- Interpret LifeLines program
+ *============================================*/
+interp_program (proc, nargs, args, nifiles, ifiles, ofile)
+STRING proc;	/* proc to call */
+INT nargs;	/* number of arguments */
+WORD *args;	/* arguments */
+INT nifiles;	/* number of program files - can be zero */
+STRING *ifiles;	/* program files */
+STRING ofile;	/* output file - can be NULL */
 {
-	FILE *ask_for_file();
-	INTERP proc;
+	LIST plist = create_list();
 	TABLE stab;
 	WORD dummy;
+	INT i;
+	STRING ifile;
+	INTERP first, parm;
 
-/* Get name of report program */
-	ifp = ask_for_file("r", "What is the name of the report program?",
-	    NULL, llprograms);
-	if (!ifp)  {
-		message(noreport);
-		return;
+   /* Get the initial list of program files */
+
+	set_list_type(plist, LISTDOFREE);
+	if (nifiles > 0) {
+		for (i = 0; i < nifiles; i++) {
+			enqueue_list(plist, strsave(ifiles[i]));
+		}
+	} else {
+		ifile = ask_for_string(qrptname, "enter string: ");
+		if (!ifile)  {
+			message(noprogram);
+			return;
+		}
+		enqueue_list(plist, strsave(ifile));
 	}
-	ofp = NULL;
 
-/* Create tables and parse program */
-	initinterp();
+   /* Parse each file in the list -- don't reparse any file */
+
+	filetab = create_table();
 	proctab = create_table();
 	globtab = create_table();
 	functab = create_table();
-	rplineno = 1;
-	if (yyparse()) return;
-	if (semerrors) {
-		wprintf("The report program contains semantic errors.\n");
+	initinterp();
+	while (!empty_list(plist)) {
+		ifile = (STRING) dequeue_list(plist);
+		if (!in_table(filetab, ifile)) {
+			insert_table(filetab, ifile, 0);
+/*wprintf("About to parse file %s.\n", ifile);/*DEBUG*/
+			parse_file(ifile, plist);
+		} else
+			stdfree(ifile);
+	}
+	if (Perrors) {
+		message("The program contains errors.\n");
 		return;
 	}
 
-/* Interpret main proc; create and pass empty symbol table */
-	if (!(proc = (INTERP) valueof(proctab, "main"))) {
-		message("The report program needs a main procedure.");
-		remove_table(proctab, NULL);
-		remove_table(globtab, NULL);
-		remove_table(functab, NULL);
+   /* Find top procedure */
+
+	if (!(first = (INTERP) valueof(proctab, proc))) {
+		message("Report program needs a starting procedure.");
+		remove_tables();
+		return;
+	}
+
+   /* Link arguments to parameters in symbol table */
+
+	parm = (INTERP) iparams(first);
+	if (nargs != num_params(parm)) {
+		mprintf("Proc %s must be called with %d (not %d) parameters.",
+			proc, num_params(parm), nargs);
+		remove_tables();
 		return;
 	}
 	stab = create_table();
-	switch (interpret((INTERP) ibody(proc), stab, &dummy)) {
+	for (i = 0; i < nargs; i++) {
+		insert_table(stab, iident(parm), args[0]);
+		parm = inext(parm);
+	}
+
+   /* Interpret top procedure */
+
+	switch (interpret((INTERP) ibody(first), stab, &dummy)) {
 	case INTOKAY:
 	case INTRETURN:
-		message("The report was successfully generated.");
+		message("The program was run successfully.");
 		break;
 	default:
-		message("The report was not generated because of errors.");
+		message("The program was not run because of errors.");
 		break;
 	}
-	remove_table(proctab, NULL);
-	remove_table(globtab, NULL);
-	remove_table(functab, NULL);
-	remove_table(stab, NULL);
+
+   /* Clean up and return */
+
+	remove_tables();
+	remove_table(stab, DONTFREE);
 	finishinterp();
-	fclose(ifp);
-	fclose(ofp);
+	if (Poutfp) fclose(Poutfp);
+	Pinfp = Poutfp = NULL;
+}
+/*============================================
+ * remove_tables - Remove interpreter's tables
+ *==========================================*/
+remove_tables ()
+{
+	remove_table(filetab, FREEKEY);
+	remove_table(proctab, DONTFREE);
+	remove_table(globtab, DONTFREE);
+	remove_table(functab, DONTFREE);
+}
+/*=======================================
+ * parse_file - Parse single program file
+ *=====================================*/
+parse_file (ifile, plist)
+STRING ifile;
+LIST plist;
+{
+	Pfname = ifile;
+	if (!ifile || *ifile == 0) return;
+	Plist = plist;
+	Pinfp = fopenpath(ifile, "r", llprograms);
+	if (!Pinfp) {
+		wprintf("Error: file \"%s\" not found.\n", ifile);
+		Perrors++;
+		return;
+	}
+	Plineno = 1;
+	yyparse();
+	fclose(Pinfp);
+}
+/*=====================================
+ * interp_main -- Interpreter main proc
+ *===================================*/
+interp_main ()
+{
+	interp_program("main", 0, NULL, 0, NULL, NULL);/*DEBUG*/
 }
 /*======================================
  * interpret -- Interpret statement list
@@ -113,27 +198,27 @@ WORD *pval;	/* possible return value */
 			poutput(iliteral(node));
 			break;
 		case IIDENT:
-			str = evaluate_iden(node, stab, &eflg);
+			str = (STRING) evaluate_iden(node, stab, &eflg);
 			if (eflg) {
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("identifier: %s\n", iident(node));
 				return FALSE;
 			}
 			poutput(str);
 			break;
 		case IBCALL:
-			str = evaluate_func(node, stab, &eflg);
+			str = (STRING) evaluate_func(node, stab, &eflg);
 			if (eflg) {
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in function: `%s'\n", iname(node));
 				return FALSE;
 			}
 			if (str) poutput(str);
 			break;
 		case IFCALL:
-			str = evaluate_ufunc(node, stab, &eflg);
+			str = (STRING) evaluate_ufunc(node, stab, &eflg);
 			if (eflg) {
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in function: `%s'\n", iname(node));
 				return FALSE;
 			}
@@ -146,7 +231,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in children loop\n");
 				return INTERROR;
 			default:
@@ -158,7 +243,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in spouses loop\n");
 				return INTERROR;
 			default:
@@ -170,7 +255,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in families loop\n");
 				return INTERROR;
 			default:
@@ -182,7 +267,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in indiset loop\n");
 				return INTERROR;
 			default:
@@ -194,7 +279,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in forindi loop\n");
 				return INTERROR;
 			default:
@@ -206,7 +291,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in forfam loop\n");
 				return INTERROR;
 			default:
@@ -218,7 +303,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in forlist loop\n");
 				return INTERROR;
 			default:
@@ -230,7 +315,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in fornotes loop\n");
 				return INTERROR;
 			default:
@@ -242,7 +327,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in fornodes loop\n");
 				return INTERROR;
 			default:
@@ -254,7 +339,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in traverse loop\n");
 				return INTERROR;
 			default:
@@ -266,7 +351,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in if statement\n");
 				return INTERROR;
 			default:
@@ -278,7 +363,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in while statement\n");
 				return INTERROR;
 			default:
@@ -290,7 +375,7 @@ WORD *pval;	/* possible return value */
 			case INTOKAY:
 				break;
 			case INTERROR:
-				wprintf(ierror, iline(node));
+				wprintf(ierror, Pfname, iline(node));
 				wprintf("in procedure call\n");
 				return INTERROR;
 			default:
@@ -325,7 +410,7 @@ INTERP node; TABLE stab; WORD *pval;
 	CACHEEL fcel, cel;
 	INTERPTYPE irc;
 	NODE fam = (NODE) eval_fam(ifamily(node), stab, &eflg, &fcel);
-	if (eflg || (fam && strcmp(ntag(fam), "FAM"))) return FALSE;
+	if (eflg || (fam && nestr(ntag(fam), "FAM"))) return FALSE;
 	if (!fam) return TRUE;
 	lock_cache(fcel);
 	FORCHILDREN(fam, chil, nchil)
@@ -362,7 +447,7 @@ INTERP node; TABLE stab; WORD *pval;
 	CACHEEL icel, scel, fcel;
 	INTERPTYPE irc;
 	NODE indi = (NODE) eval_indi(iprinc(node), stab, &eflg, &icel);
-	if (eflg || (indi && strcmp(ntag(indi), "INDI"))) return FALSE;
+	if (eflg || (indi && nestr(ntag(indi), "INDI"))) return FALSE;
 	if (!indi) return TRUE;
 	lock_cache(icel);
 	FORSPOUSES(indi, spouse, fam, nspouses)
@@ -403,10 +488,10 @@ INTERP node; TABLE stab; WORD *pval;
 	CACHEEL icel, fcel, scel;
 	INTERPTYPE irc;
 	NODE indi = (NODE) eval_indi(iprinc(node), stab, &eflg, &icel);
-	if (eflg || (indi && strcmp(ntag(indi), "INDI"))) return FALSE;
+	if (eflg || (indi && nestr(ntag(indi), "INDI"))) return FALSE;
 	if (!indi) return TRUE;
 	lock_cache(icel);
-	FORFAMILIES(indi, fam, spouse, nfamilies)
+	FORFAMSS(indi, fam, spouse, nfamilies)
 		insert_table(stab, ifamvar(node),
 		    (WORD) (fcel = fam_to_cacheel(fam)));
 		insert_table(stab, ispouse(node),
@@ -429,7 +514,7 @@ INTERP node; TABLE stab; WORD *pval;
 			return irc;
 		}
 c:	;
-	ENDFAMILIES
+	ENDFAMSS
 	unlock_cache(icel);
 	return INTOKAY;
 }
@@ -670,11 +755,11 @@ INTERP node; TABLE stab; WORD *pval;
 	}
 	if (arg || parm) {
 		wprintf("``%s'': mismatched args and params\n", iname(node));
-		remove_table(newtab, NULL);
+		remove_table(newtab, DONTFREE);
 		return INTERROR;
 	}
 	irc = interpret((INTERP) ibody(proc), newtab, pval);
-	remove_table(newtab, NULL);
+	remove_table(newtab, DONTFREE);
 	switch (irc) {
 	case INTRETURN:
 	case INTOKAY:
