@@ -2,7 +2,7 @@
  * interface.c -- Interface between LifeLines and BTREE database
  * Copyright(c) 1992-4 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 18 Aug 93
- *   3.0.0 - 20 Aug 94
+ *   3.0.0 - 20 Aug 94    3.0.2 - 09 Nov 94
  *============================================================*/
 
 #include "standard.h"
@@ -17,7 +17,7 @@ STRING retrieve_record (key, plen)
 STRING key;
 INT *plen;
 {
-	return getrecord(BTR, str2rkey(key), plen);
+	return (STRING) getrecord(BTR, str2rkey(key), plen);
 }
 /*=========================================
  * store_record -- Store record in database
