@@ -2,7 +2,7 @@
  * merge.c -- Merge persons and families
  * Copyright(c) 1992-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 01 Sep 93
- *   3.0.0 - 23 Sep 94
+ *   3.0.0 - 23 Sep 94    3.0.2 - 13 Dec 94
  *===========================================================*/
 
 #include "standard.h"
@@ -13,7 +13,6 @@
 
 extern STRING iredit, cfpmrg, nopmrg, noqmrg, noxmrg, nofmrg;
 extern STRING dhusb,  dwife,  cffmrg, fredit, badata, ronlym;
-extern TRANTABLE tran_tables[];
 
 /*================================================================
  * merge_two_indis -- Merge first person to second; data from both
@@ -22,11 +21,11 @@ extern TRANTABLE tran_tables[];
 NODE merge_two_indis (indi1, indi2)
 NODE indi1, indi2;
 {
-	NODE name1, sex1, body1, famc1, fams1;
-	NODE name2, sex2, body2, famc2, fams2;
-	NODE indi3, name3, sex3, body3, famc3, fams3;
-	NODE indi4, name4, sex4, body4, famc4, fams4;
-	NODE fam, husb, wife, chil, rest;
+	NODE name1, refn1, sex1, body1, famc1, fams1;
+	NODE name2, refn2, sex2, body2, famc2, fams2;
+	NODE indi3, name3, refn3, sex3, body3, famc3, fams3;
+	NODE indi4, name4, refn4, sex4, body4, famc4, fams4;
+	NODE fam, husb, wife, chil, rest, fref;
 	NODE curs1, curs2, node, this, prev, next, old, new;
 	TRANTABLE tti = tran_tables[MEDIN], tto = tran_tables[MINED];
 	FILE *fp;
@@ -52,10 +51,12 @@ NODE indi1, indi2;
 /* Check restrictions on persons */
 	famc1 = FAMC(indi1);
 	famc2 = FAMC(indi2);
-	if (famc1 && famc2 && !eqstr(nval(famc1), nval(famc2))) {
+#if 0
+	if (famc1 && famc2 && nestr(nval(famc1), nval(famc2))) {
 		message(noqmrg);
 		return NULL;
 	}
+#endif
 	fams1 = FAMS(indi1);
 	fams2 = FAMS(indi2);
 	if (fams1 && fams2 && SEX(indi1) != SEX(indi2)) {
@@ -64,28 +65,34 @@ NODE indi1, indi2;
 	}
 
 /* Split persons */
-	split_indi(indi1, &name1, &sex1, &body1, &famc1, &fams1);
-	split_indi(indi2, &name2, &sex2, &body2, &famc2, &fams2);
+	split_indi(indi1, &name1, &refn1, &sex1, &body1, &famc1, &fams1);
+	split_indi(indi2, &name2, &refn2, &sex2, &body2, &famc2, &fams2);
 	ASSERT(name1 && name2);
 	sx2 = SEX_UNKNOWN;
 	if (fams1) sx2 = val_to_sex(sex1);
 	if (fams2) sx2 = val_to_sex(sex2);
 
+/*CONDITION 1 -- 1 & 2 split */
+
 /* Construct first version of merged person */
 	ASSERT(fp = fopen(editfile, "w"));
 	indi3 = copy_nodes(indi2, TRUE, TRUE);
 	name3 = unique_nodes(name1, name2);
-	sex3 = unique_nodes(sex1, sex2);
+	refn3 = unique_nodes(refn1, refn2);
+	sex3  = unique_nodes(sex1, sex2);
 	body3 = unique_nodes(body1, body2);
 	famc3 = unique_nodes(famc1, famc2);
 	fams3 = unique_nodes(fams1, fams2);
 	write_nodes(0, fp, tto, indi3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, name3, TRUE, TRUE, TRUE);
+	write_nodes(1, fp, tto, refn3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, sex3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, body3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, famc3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, fams3, TRUE, TRUE, TRUE);
 	fclose(fp);
+	join_indi(indi3, name3, refn3, sex3, body3, famc3, fams3);
+/*CONDITION 2 -- 3 (init combined) created and joined*/
 
 /* Have user edit merged person */
 	do_edit();
@@ -98,7 +105,7 @@ NODE indi1, indi2;
 			} 
 			break;
 		}
-		if (!valid_indi_tree(indi4, &msg, sx2, indi3, famc3, fams3)) {
+		if (!valid_indi_tree(indi4, &msg, indi3)) {
 			if (ask_yes_or_no_msg(msg, iredit)) {
 				do_edit();
 				continue;
@@ -109,26 +116,25 @@ NODE indi1, indi2;
 		}
 		break;
 	}
-	join_indi(indi3, name3, sex3, body3, famc3, fams3);
 	free_nodes(indi3);
 
 /* Have user confirm changes */
 	if (!indi4 || !ask_yes_or_no(cfpmrg)) {
 		if (indi4) free_nodes(indi4);
-		join_indi(indi1, name1, sex1, body1, famc1, fams1);
-		join_indi(indi2, name2, sex2, body2, famc2, fams2);
+		join_indi(indi1, name1, refn1, sex1, body1, famc1, fams1);
+		join_indi(indi2, name2, refn2, sex2, body2, famc2, fams2);
 		return NULL;
 	}
 
 /* Modify families that have persons as children */
-	split_indi(indi4, &name4, &sex4, &body4, &famc4, &fams4);
+	split_indi(indi4, &name4, &refn4, &sex4, &body4, &famc4, &fams4);
 	curs1 = famc1;
 	while (curs1) {
 		curs2 = famc2;
-		while (curs2 && !eqstr(nval(curs1), nval(curs2)))
+		while (curs2 && nestr(nval(curs1), nval(curs2)))
 			curs2 = nsibling(curs2);
 		fam = key_to_fam(rmvat(nval(curs1)));
-		split_fam(fam, &husb, &wife, &chil, &rest);
+		split_fam(fam, &fref, &husb, &wife, &chil, &rest);
 		prev = NULL;
 		this = chil;
 
@@ -149,7 +155,7 @@ NODE indi1, indi2;
 					this = nsibling(this);
 				}
 			}
-			join_fam(fam, husb, wife, chil, rest);
+			join_fam(fam, fref, husb, wife, chil, rest);
 			fam_to_dbase(fam);
 
 /* Only first was child; make family refer to second */
@@ -162,7 +168,7 @@ NODE indi1, indi2;
 				prev = this;
 				this = nsibling(this);
 			}
-			join_fam(fam, husb, wife, chil, rest);
+			join_fam(fam, fref, husb, wife, chil, rest);
 			fam_to_dbase(fam);
 		}
 		curs1 = nsibling(curs1);
@@ -172,7 +178,7 @@ NODE indi1, indi2;
 	this = fams1;
 	while (this) {
 		fam = key_to_fam(rmvat(nval(this)));
-		split_fam(fam, &husb, &wife, &chil, &rest);
+		split_fam(fam, &fref, &husb, &wife, &chil, &rest);
 		if (sx2 == SEX_MALE) {
 			ASSERT(husb);
 			stdfree(nval(husb));
@@ -182,13 +188,13 @@ NODE indi1, indi2;
 			stdfree(nval(wife));
 			nval(wife) = (STRING) strsave(nxref(indi2));
 		}
-		join_fam(fam, husb, wife, chil, rest);
+		join_fam(fam, fref, husb, wife, chil, rest);
 		fam_to_dbase(fam);
 		this = nsibling(this);
 	}
 
 /* Remove unneeded nodes and delete first person */
-	join_indi(indi1, name1, sex1, body1, NULL, NULL);
+	join_indi(indi1, name1, refn1, sex1, body1, NULL, NULL);
 	delete_indi(indi1, FALSE);
 	free_nodes(famc1);
 	free_nodes(fams1);
@@ -200,8 +206,9 @@ NODE indi1, indi2;
 /* Put merged person in database */
 	old = name2;
 	new = copy_nodes(name4, TRUE, TRUE);
-	join_indi(indi2, name4, sex4, body4, famc4, fams4);
+	join_indi(indi2, name4, refn4, sex4, body4, famc4, fams4);
 	remove_duplicate_names(&old, &new);
+	resolve_links(indi2);
 	indi_to_dbase(indi2);
 	key = rmvat(nxref(indi2));
 	for (node = old; node; node = nsibling(node))
@@ -221,8 +228,9 @@ NODE merge_two_fams (fam1, fam2)
 NODE fam1, fam2;
 {
 	NODE husb1, wife1, chil1, rest1, husb2, wife2, chil2, rest2;
-	NODE fam3, husb3, wife3, chil3, rest3;
-	NODE fam4, husb4, wife4, chil4, rest4;
+	NODE fref1, fref2;
+	NODE fam3, husb3, wife3, chil3, rest3, fref3;
+	NODE fam4, husb4, wife4, chil4, rest4, fref4;
 	TRANTABLE tti = tran_tables[MEDIN], tto = tran_tables[MINED];
 	FILE *fp;
 	STRING msg;
@@ -241,29 +249,33 @@ NODE fam1, fam2;
 	}
 
 /* Check restrictions on families */
-	split_fam(fam1, &husb1, &wife1, &chil1, &rest1);
-	split_fam(fam2, &husb2, &wife2, &chil2, &rest2);
-	if (husb1 && husb2 && !eqstr(nval(husb1), nval(husb2))) {
+	split_fam(fam1, &fref1, &husb1, &wife1, &chil1, &rest1);
+	split_fam(fam2, &fref2, &husb2, &wife2, &chil2, &rest2);
+#if 0
+	if (husb1 && husb2 && nestr(nval(husb1), nval(husb2))) {
 		message(dhusb);
-		join_fam(fam1, husb1, wife1, chil1, rest1);
-		join_fam(fam2, husb2, wife2, chil2, rest2);
+		join_fam(fam1, fref1, husb1, wife1, chil1, rest1);
+		join_fam(fam2, fref2, husb2, wife2, chil2, rest2);
 		return NULL;
 	}
-	if (wife1 && wife2 && !eqstr(nval(wife1), nval(wife2))) {
+	if (wife1 && wife2 && nestr(nval(wife1), nval(wife2))) {
 		message(dwife);
-		join_fam(fam1, husb1, wife1, chil1, rest1);
-		join_fam(fam2, husb2, wife2, chil2, rest2);
+		join_fam(fam1, fref1, husb1, wife1, chil1, rest1);
+		join_fam(fam2, fref2, husb2, wife2, chil2, rest2);
 		return NULL;
 	}
+#endif
 
 /* Create merged file with both families together */
 	ASSERT(fp = fopen(editfile, "w"));
 	fam3 = copy_nodes(fam2, TRUE, TRUE);
+	fref3 = unique_nodes(fref1, fref2);
 	husb3 = unique_nodes(husb1, husb2);
 	wife3 = unique_nodes(wife1, wife2);
 	rest3 = unique_nodes(rest1, rest2);
 	chil3 = sort_children(chil1, chil2);
 	write_nodes(0, fp, tto, fam3, TRUE, TRUE, TRUE);
+	write_nodes(1, fp, tto, fref3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, husb3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, wife3, TRUE, TRUE, TRUE);
 	write_nodes(1, fp, tto, rest3, TRUE, TRUE, TRUE);
@@ -292,17 +304,17 @@ NODE fam1, fam2;
 		}
 		break;
 	}
-	join_fam(fam3, husb3, wife3, rest3, chil3);
+	join_fam(fam3, fref3, husb3, wife3, rest3, chil3);
 	free_nodes(fam3);
 
 /* Have user confirm changes */
 	if (!fam4 || !ask_yes_or_no(cffmrg)) {
 		if (fam4) free_nodes(fam4);
-		join_fam(fam1, husb1, wife1, chil1, rest1);
-		join_fam(fam2, husb2, wife2, chil2, rest2);
+		join_fam(fam1, fref1, husb1, wife1, chil1, rest1);
+		join_fam(fam2, fref2, husb2, wife2, chil2, rest2);
 		return NULL;
 	}
-	split_fam(fam4, &husb4, &wife4, &chil4, &rest4);
+	split_fam(fam4, &fref4, &husb4, &wife4, &chil4, &rest4);
 
  /* Modify links between persons and families */
 #define CHUSB 1
@@ -313,7 +325,7 @@ NODE fam1, fam2;
 	merge_fam_links (fam1, fam2, chil1, chil2, CCHIL);
 
 /* Update database with second family; remove first */
-	join_fam(fam4, husb2, wife2, chil2, rest2);
+	join_fam(fam4, fref2, husb2, wife2, chil2, rest2);
 	free_nodes(fam4);
 	nchild(fam1) = NULL;
 	delete_fam(fam1);
@@ -321,7 +333,8 @@ NODE fam1, fam2;
 	free_nodes(wife1);
 	free_nodes(chil1);
 	free_nodes(rest1);
-	join_fam(fam2, husb4, wife4, chil4, rest4);
+	join_fam(fam2, fref4, husb4, wife4, chil4, rest4);
+	resolve_links(fam2);
 	fam_to_dbase(fam2);
 	return fam2;
 }
@@ -336,12 +349,12 @@ NODE root1, root2;
 	NODE chil1, chil2;
 	if (!root1 && !root2) return TRUE;
 	if (!root1 || !root2) return FALSE;
-	if (strcmp(ntag(root1), ntag(root2))) return FALSE;
+	if (nestr(ntag(root1), ntag(root2))) return FALSE;
 	str1 = nval(root1);
 	str2 = nval(root2);
 	if (str1 && !str2) return FALSE;
 	if (str2 && !str1) return FALSE;
-	if (str1 && str2 && strcmp(str1, str2)) return FALSE;
+	if (str1 && str2 && nestr(str1, str2)) return FALSE;
 	nchil1 = node_list_length(nchild(root1));
 	nchil2 = node_list_length(nchild(root2));
 	if (nchil1 != nchil2) return FALSE;
@@ -469,14 +482,14 @@ NODE fam1, fam2, list1, list2;
 INT code;
 {
 	NODE curs1, curs2, prev, this, next, first;
-	NODE indi, name, sex, body, famc, fams;
+	NODE indi, name, refn, sex, body, famc, fams;
 	curs1 = list1;
 	while (curs1) {
 		curs2 = list2;
-		while (curs2 && !eqstr(nval(curs1), nval(curs2)))
+		while (curs2 && nestr(nval(curs1), nval(curs2)))
 			curs2 = nsibling(curs2);
 		indi = key_to_indi(rmvat(nval(curs1)));
-		split_indi(indi, &name, &sex, &body, &famc, &fams);
+		split_indi(indi, &name, &refn, &sex, &body, &famc, &fams);
 		prev = NULL;
 		if (code == CHUSB || code == CWIFE)
 			first = this = fams;
@@ -516,7 +529,7 @@ INT code;
 			fams = first;
 		else
 			famc = first;
-		join_indi(indi, name, sex, body, famc, fams);
+		join_indi(indi, name, refn, sex, body, famc, fams);
 		indi_to_dbase(indi);
 		curs1 = nsibling(curs1);
 	}
@@ -537,16 +550,16 @@ NODE chil1, chil2;
 	while (copy1 && copy2) {
 		if (int1 == 1) {
 			kid1 = key_to_indi(rmvat(nval(copy1)));
-			year1 = event_to_date(BIRT(kid1), TRUE);
+			year1 = event_to_date(BIRT(kid1), NULL, TRUE);
 			if (!year1)
-				year1 = event_to_date(BAPT(kid1), TRUE);
+				year1 = event_to_date(BAPT(kid1), NULL, TRUE);
 			int1 = year1 ? atoi(year1) : 0;
 		}
 		if (int2 == 1) {
 			kid2 = key_to_indi(rmvat(nval(copy2)));
-			year2 = event_to_date(BIRT(kid2), TRUE);
+			year2 = event_to_date(BIRT(kid2), NULL, TRUE);
 			if (!year2)
-				year2 = event_to_date(BAPT(kid2), TRUE);
+				year2 = event_to_date(BAPT(kid2), NULL, TRUE);
 			int2 = year2 ? atoi(year2) : 0;
 		}
 		if (int1 < int2) {
@@ -591,7 +604,7 @@ NODE list1, list2;
 	curs1 = copy1;
 	while (curs1) {
 		curs2 = list2;
-		while (curs2 && !eqstr(nval(curs1), nval(curs2)))
+		while (curs2 && nestr(nval(curs1), nval(curs2)))
 			curs2 = nsibling(curs2);
 		if (curs2) {
 			next1 = nsibling(curs1);
