@@ -2,7 +2,7 @@
  * more.c -- More builtin functions of report interpreter
  * Copyright(c) 1992-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 26 Sep 93
- *   3.0.0 - 28 Jun 94
+ *   3.0.0 - 28 Jun 94    3.0.2 - 04 Apr 95
  *===========================================================*/
 
 #include <stdio.h>
@@ -20,10 +20,10 @@ static INT index();
 static compute_pi();
 
 /*==============================================================
- * _extractnames -- Extract name parts from person or NAME node.
+ * __extractnames -- Extract name parts from person or NAME node.
  *   usage: extractnames(NODE, LIST, VARB, VARB) -> VOID
  *============================================================*/
-WORD _extractnames (node, stab, eflg)
+WORD __extractnames (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP nexp = (INTERP) ielist(node);
@@ -50,10 +50,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return NULL;
 }
 /*===============================================================
- * _extractplaces -- Extract place parts from event or PLAC NODE.
+ * __extractplaces -- Extract place parts from event or PLAC NODE.
  *   usage: extractplaces(NODE, LIST, VARB) -> VOID
  *=============================================================*/
-WORD _extractplaces (node, stab, eflg)
+WORD __extractplaces (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP nexp = (INTERP) ielist(node);
@@ -78,10 +78,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return NULL;
 }
 /*===============================================================
- * _extracttokens -- Extract tokens from a STRING value
+ * __extracttokens -- Extract tokens from a STRING value
  *   usage: extracttokens(STRING, LIST, VARB, STRING) -> VOID
  *=============================================================*/
-WORD _extracttokens (node, stab, eflg)
+WORD __extracttokens (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP sexp = (INTERP) ielist(node);
@@ -105,22 +105,25 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	assign_iden(stab, iident(lvar), len);
 	return NULL;
 }
-/*===================================
- * _database -- Return database name.
- *   usage: database() -> STRING
- *=================================*/
-WORD _database (node, stab, eflg)
+/*====================================
+ * __database -- Return database name
+ *   usage: database([BOOL]) -> STRING
+ *==================================*/
+WORD __database (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
-	extern STRING btreepath;
+	extern STRING readpath, lastpathname();
+	BOOLEAN full = FALSE;
 	*eflg = FALSE;
-	return btreepath;
+	if (ielist(node)) full = (BOOLEAN) evaluate(ielist(node), stab, eflg);
+	if (*eflg) return NULL;
+	return (WORD) (full ? readpath : lastpathname(readpath));
 }
-/*============================================
- * _index -- Find nth occurrence of substring.
+/*===========================================
+ * __index -- Find nth occurrence of substring
  *   usage: index(STRING, STRING, INT) -> INT
- *==========================================*/
-WORD _index (node, stab, eflg)
+ *=========================================*/
+WORD __index (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP arg = (INTERP) ielist(node);
@@ -136,10 +139,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return (WORD) index(str, sub, num);
 }
 /*===============================================
- * _substring -- Find substring of string.
+ * __substring -- Find substring of string.
  *   usage: substring(STRING, INT, INT) -> STRING
  *=============================================*/
-WORD _substring (node, stab, eflg)
+WORD __substring (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP arg = (INTERP) ielist(node);
@@ -162,7 +165,11 @@ static INT index (str, sub, num)
 STRING str, sub;
 INT num;
 {
-        INT n = strlen(str), m = strlen(sub), q = 0, i, found = 0;
+        INT i, n, m, q = 0, found = 0;
+
+	if (!str || !sub || *str == 0 || *sub == 0) return 0;
+        n = strlen(str);
+	m = strlen(sub);
         compute_pi(sub);
         for (i = 1; i <= n; i++) {
                 while (q > 0 && sub[q] != str[i-1])
@@ -208,7 +215,7 @@ INT i, j;
  * chooseindi -- Have user choose person from set.
  *   usage: chooseindi(SET) -> INDI
  *==============================================*/
-WORD _chooseindi (node, stab, eflg)
+WORD __chooseindi (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INDISEQ seq = (INDISEQ) evaluate(ielist(node), stab, eflg);
@@ -222,7 +229,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * choosesubset -- Have user choose subset from set.
  *   usage: choosesubset(SET) -> SET
  *================================================*/
-WORD _choosesubset (node, stab, eflg)
+WORD __choosesubset (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	STRING msg;
@@ -238,7 +245,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * choosechild -- Have user choose child of person or family.
  *   usage: choosechild(INDI|FAM) -> INDI
  *=========================================================*/
-WORD _choosechild (node, stab, eflg)
+WORD __choosechild (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	CACHEEL cel = (CACHEEL) evaluate(ielist(node), stab, eflg);
@@ -272,7 +279,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * choosespouse -- Have user choose spouse of person.
  *   usage: choosespouse(INDI) -> INDI
  *=================================================*/
-WORD _choosespouse (node, stab, eflg)
+WORD __choosespouse (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE indi = eval_indi(ielist(node), stab, eflg, NULL);
@@ -289,35 +296,35 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * choosefam -- Have user choose family of person.
  *   usage: choosefam (INDI) -> FAM
  *==============================================*/
-WORD _choosefam (node, stab, eflg)
+WORD __choosefam (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE fam, indi = eval_indi(ielist(node), stab, eflg, NULL);
 	INDISEQ seq;
 	if (*eflg) return NULL;
-	seq = indi_to_families(indi);
+	seq = indi_to_families(indi, TRUE);
 	if (!seq || length_indiseq(seq) < 1) return NULL;
 	fam = format_and_choose_indi(seq, TRUE, TRUE, TRUE, ifone, notone);
 	remove_indiseq(seq, FALSE);
 	if (!fam) return NULL;
 	return (WORD) fam_to_cacheel(fam);
 }
-/*=====================================================
- * menuchoose -- Have user choose from list of options.
+/*====================================================
+ * menuchoose -- Have user choose from list of options
  *   usage: menuchoose (LIST [,STRING]) -> INT
- *===================================================*/
-WORD _menuchoose (node, stab, eflg)
+ *==================================================*/
+WORD __menuchoose (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INT i, len;
 	STRING msg, *strngs;
-	STRING ttl = "Please choose from the following list.";
+	STRING ttl = (STRING) "Please choose from the following list.";
 	INTERP arg = (INTERP) ielist(node);
 	LIST list = (LIST) evaluate(arg, stab, eflg);
 	if (!list || *eflg || length_list(list) < 1) return (WORD) 0;
 	msg = NULL;
 	arg = (INTERP) inext(arg);
-	if (arg) msg = evaluate(arg, stab, eflg);
+	if (arg) msg = (STRING) evaluate(arg, stab, eflg);
 	if (*eflg) return (WORD) 0;
 	if (msg && *msg) ttl = msg;
 	len = length_list(list);
@@ -331,10 +338,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return (WORD) (i + 1);
 }
 /*=================================
- * system -- Run shell command.
+ * system -- Run shell command
  *   usage: system (STRING) -> VOID
  *===============================*/
-WORD _system (node, stab, eflg)
+WORD __system (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	STRING cmd = (STRING) evaluate(ielist(node), stab, eflg);
@@ -344,11 +351,11 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	system(cmd);
 	return NULL;
 }
-/*==============================================
- * firstindi -- Return first person in database.
+/*=============================================
+ * firstindi -- Return first person in database
  *   usage: firstindi() -> INDI
- *============================================*/
-WORD _firstindi (node, stab, eflg)
+ *===========================================*/
+WORD __firstindi (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE indi;
@@ -373,7 +380,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * nextindi -- Return next person in database.
  *   usage: nextindi(INDI) -> INDI
  *==========================================*/
-WORD _nextindi (node, stab, eflg)
+WORD __nextindi (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE indi = eval_indi(ielist(node), stab, eflg, NULL);
@@ -400,7 +407,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * previndi -- Return previous person in database.
  *   usage: previndi(INDI) -> INDI
  *==============================================*/
-WORD _previndi (node, stab, eflg)
+WORD __previndi (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE indi = eval_indi(ielist(node), stab, eflg, NULL);
@@ -427,7 +434,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * lastindi -- Return last person in database.
  *   usage: lastindi() -> INDI
  *==========================================*/
-WORD _lastindi (node, stab, eflg)
+WORD __lastindi (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 }
@@ -435,7 +442,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * firstfam -- Return first family in database.
  *   usage: firstfam() -> FAM
  *===========================================*/
-WORD _firstfam (node, stab, eflg)
+WORD __firstfam (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE fam;
@@ -460,7 +467,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * nextfam -- Return next family in database.
  *   usage: nextfam(FAM) -> FAM
  *=========================================*/
-WORD _nextfam (node, stab, eflg)
+WORD __nextfam (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE fam = eval_fam(ielist(node), stab, eflg, NULL);
@@ -487,7 +494,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * prevfam -- Return previous family in database.
  *   usage: prevfam(FAM) -> FAM
  *=============================================*/
-WORD _prevfam (node, stab, eflg)
+WORD __prevfam (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE fam = eval_fam(ielist(node), stab, eflg, NULL);
@@ -514,7 +521,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * lastfam -- Return last family in database.
  *   usage: lastfam() -> FAM
  *==========================================*/
-WORD _lastfam (node, stab, eflg)
+WORD __lastfam (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 }
@@ -524,7 +531,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  *  usage: dereference(STRING) -> NODE
  *  NOTE: persons and families NOT cached!
  *=============================================*/
-WORD _getrecord (node, stab, eflg)
+WORD __getrecord (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	STRING key = (STRING) evaluate(ielist(node), stab, eflg);
@@ -545,7 +552,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * freerecord -- Free GEDCOM node tree from getrecord.
  *  usage: getrecord(STRING) -> NODE
  *==================================================*/
-WORD _freerecord (node, stab, eflg)
+WORD __freerecord (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 }
@@ -553,7 +560,7 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
  * reference -- Check if STRING is record reference.
  *  usage: reference(STRING) -> BOOLEAN
  *================================================*/
-WORD _reference (node, stab, eflg)
+WORD __reference (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	STRING key = (STRING) evaluate(ielist(node), stab, eflg);
@@ -562,10 +569,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	    (key[strlen(key)-1] == '@'));
 }
 /*=========================================
- * _rjustify -- Right justify string value.
+ * __rjustify -- Right justify string value
  *   usage: rjustify(STRING, INT) -> STRING
  *=======================================*/
-WORD _rjustify (node, stab, eflg)
+WORD __rjustify (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	INTERP sarg = (INTERP) ielist(node);
@@ -576,11 +583,11 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	if (*eflg || !str) return NULL;
 	len = (INT) evaluate(larg, stab, eflg);
 	if (*eflg || len < 1 || len > 512) return NULL;
-	return rightjustify(str, len);
+	return (WORD) rightjustify(str, len);
 }
-/*============================================
- * rightjustify -- Right justify string value.
- *==========================================*/
+/*===========================================
+ * rightjustify -- Right justify string value
+ *=========================================*/
 STRING rightjustify (str, len)
 STRING str;
 INT len;
@@ -601,10 +608,10 @@ INT len;
 	return new;
 }
 /*=========================================
- * _lock -- Lock person or family in memory
+ * __lock -- Lock person or family in memory
  *   usage: lock(INDI|FAM) -> VOID
  *=======================================*/
-WORD _lock (node, stab, eflg)
+WORD __lock (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	CACHEEL cel = (CACHEEL) evaluate(ielist(node), stab, eflg);
@@ -612,10 +619,10 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return NULL;
 }
 /*===============================================
- * _unlock -- Unlock person or family from memory
+ * __unlock -- Unlock person or family from memory
  *   usage: unlock(INDI|FAM) -> VOID
  *=============================================*/
-WORD _unlock (node, stab, eflg)
+WORD __unlock (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	CACHEEL cel = (CACHEEL) evaluate(ielist(node), stab, eflg);
@@ -623,13 +630,41 @@ INTERP node; TABLE stab; BOOLEAN *eflg;
 	return NULL;
 }
 /*==========================================
- * _savenode -- Save GEDCOM tree permanently
+ * __savenode -- Save GEDCOM tree permanently
  *   usage: savenode(NODE) -> NODE
  *========================================*/
-WORD _savenode (node, stab, eflg)
+WORD __savenode (node, stab, eflg)
 INTERP node; TABLE stab; BOOLEAN *eflg;
 {
 	NODE this = (NODE) evaluate(ielist(node), stab, eflg);
 	if (!this) return NULL;
 	return (WORD) copy_nodes(this, TRUE, TRUE);
 }
+/*===================================================
+ * __genindiset -- Generate set of persons from a name
+ *   usage: genindiset(STRING, SET) -> VOID
+ *=================================================*/
+WORD __genindiset (node, stab, eflg)
+INTERP node; TABLE stab; BOOLEAN *eflg;
+{
+	INTERP arg = (INTERP) ielist(node);
+	STRING name = (STRING) evaluate(arg, stab, eflg);
+	if (!name || *name == 0 || *eflg) return NULL;
+	arg = inext(arg);
+	*eflg = TRUE;
+	if (!iistype(arg, IIDENT)) return NULL;
+	*eflg = FALSE;
+	assign_iden(stab, iident(arg), name_to_indiseq(name));
+	return NULL;
+}
+/*=================================================
+ * __version -- Return the LifeLines version string
+ *   usage: version() -> STRING
+ *===============================================*/
+WORD __version (node, stab, eflg)
+INTERP node; TABLE stab; BOOLEAN *eflg;
+{
+	extern STRING version;
+	*eflg = FALSE;
+	return (WORD) version;
+}
