@@ -1,14 +1,15 @@
 /*=============================================================
- * names.c -- Handle name indexing
+ * names.c -- Handle name values and name indexing
  * Copyright(c) 1992-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 02 Sep 93
- *   3.0.0 - 13 Sep 94
+ *   3.0.0 - 13 Sep 94    3.0.2 - 31 Dec 94
  *===========================================================*/
 
 #include "standard.h"
 #include "table.h"
 #include "btree.h"
 #include "gedcom.h"
+#include "translat.h"
 
 extern BTREE BTR;
 
@@ -148,17 +149,17 @@ STRING getsurname (name)
 STRING name;	/* GEDCOM name */
 {
 	INT c;
-	static char buffer[3][MAXLINELEN+1];
+	static unsigned char buffer[3][MAXLINELEN+1];
 	static INT dex = 0;
 	STRING p, surname;
 	if (++dex > 2) dex = 0;
 	p = surname = buffer[dex];
 	while ((c = *name++) && c != '/')
 		;
-	if (c == 0) return "____";
+	if (c == 0) return (STRING) "____";
 	while (iswhite(c = *name++))
 		;
-	if (c == 0 || c == '/' || !isletter(c)) return "____";
+	if (c == 0 || c == '/' || !isletter(c)) return (STRING) "____";
 	*p++ = c;
 	while ((c = *name++) && c != '/')
 		*p++ = c;
@@ -175,7 +176,7 @@ STRING name;	/* GEDCOM name */
 	while (TRUE) {
 		while (iswhite(c = *name++))
 			;
-		if (isletter(c)) return toupper(c);
+		if (isletter(c)) return ll_toupper(c);
 		if (c == 0) return '$';
 		if (c != '/') return '$';
 		while ((c = *name++) && c != '/')
@@ -190,15 +191,15 @@ STRING name;	/* GEDCOM name */
 STRING soundex (name)
 STRING name;	/* surname */
 {
-	static char scratch[MAXNAMELEN+2];
+	static unsigned char scratch[MAXNAMELEN+2];
 	STRING p = name, q = scratch;
 	INT c, i, j;
-	if (!name || strlen(name) > MAXNAMELEN || !strcmp(name, "____"))
-		return "Z999";
+	if (!name || strlen(name) > MAXNAMELEN || eqstr(name, "____"))
+		return (STRING) "Z999";
 	p = name;
 	q = scratch;
 	while (c = *p++)
-		*q++ = toupper(c);
+		*q++ = ll_toupper(c);
 	*q = 0;
 	p = q = &scratch[1];
 	i = 1;
@@ -261,7 +262,7 @@ STRING key;	/* person's INDI key */
 	(void) getnamerec(name);
 	for (i = 0; i < NRcount; i++) {
 		if (!strncmp(rkey.r_rkey, NRkeys[i].r_rkey, 8) &&
-		    !strcmp(name, NRnames[i]))
+		    eqstr(name, NRnames[i]))
 			return TRUE;
 	}
 	NRkeys[NRcount] = rkey;
@@ -310,7 +311,7 @@ STRING key;	/* person's INDI key */
 	found = FALSE;
 	for (i = 0; i < NRcount; i++) {
 		if (!strncmp(rkey.r_rkey, NRkeys[i].r_rkey, 8) &&
-		    !strcmp(name, NRnames[i])) {
+		    eqstr(name, NRnames[i])) {
 			found = TRUE;
 			break;
 		}
@@ -417,9 +418,9 @@ STRING out;	/* superstring of words */
 		return;
 	}
 	while (TRUE) {
-		*out++ = toupper(c);
+		*out++ = ll_toupper(c);
 		while ((c = *in++) && c != '/' && chartype(c) != WHITE) {
-			if (chartype(c) == LETTER) *out++ = toupper(c);
+			if (chartype(c) == LETTER) *out++ = ll_toupper(c);
 		}
 		if (c == 0) {
 			*out++ = 0; *out = 0;
@@ -495,16 +496,16 @@ BOOLEAN exact;	/* unused! */
 int namecmp (name1, name2)
 STRING name1, name2;
 {
-	char sqz1[MAXNAMELEN], sqz2[MAXNAMELEN];
+	unsigned char sqz1[MAXNAMELEN], sqz2[MAXNAMELEN];
 	STRING p1 = sqz1, p2 = sqz2;
-	INT r = strcmp(getsurname(name1), getsurname(name2));
+	INT r = nestr(getsurname(name1), getsurname(name2));
 	if (r) return r;
 	r = getfinitial(name1) - getfinitial(name2);
 	if (r) return r;
 	cmpsqueeze(name1, p1);
 	cmpsqueeze(name2, p2);
 	while (*p1 && *p2) {
-		r = strcmp(p1, p2);
+		r = nestr(p1, p2);
 		if (r) return r;
 		p1 += strlen(p1) + 1;
 		p2 += strlen(p2) + 1;
@@ -540,7 +541,7 @@ STRING givens (name)
 STRING name;
 {
 	INT c;
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING out = scratch;
 	while (name = nextpiece(name)) {
 		while (TRUE) {
@@ -622,7 +623,7 @@ static name_to_parts (name, parts)
 STRING name;	/* GEDCOM name */
 STRING *parts;
 {
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING p = scratch;
 	INT c, i = 0;
 	ASSERT(strlen(name) <= MAXNAMELEN);
@@ -657,7 +658,7 @@ static STRING parts_to_name (parts)
 STRING *parts;
 {
 	INT i;
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING p = scratch;
 	for (i = 0; i < MAXPARTS; i++) {
 		if (!parts[i]) continue;
@@ -677,45 +678,46 @@ STRING *parts;
 STRING upsurname (name)
 STRING name;
 {
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING p = scratch;
 	INT c;
 	while ((c = *p++ = *name++) && c != '/')
 		;
 	if (c == 0) return scratch;
 	while ((c = *name++) && c != '/')
-		*p++ = toupper(c);
+		*p++ = ll_toupper(c);
 	*p++ = c;
 	if (c == 0) return scratch;
 	while (c = *p++ = *name++)
 		;
 	return scratch;
 }
-/*================================================================
- * manip_name - Converts GEDCOM names to various formats; second
- *   parameter specifies surname in upper case; third specifies
- *   surname first in string and separated from givens by a comma;
- *   fourth parameter specifies max length the output string
- *==============================================================*/
-STRING manip_name (name, caps, reg, len)
+/*==================================================
+ * manip_name - Convert GEDCOM name to various forms
+ *================================================*/
+STRING manip_name (name, tt, caps, reg, len)
 STRING name;	/* name */
+TRANTABLE tt;	/* translation table */
 BOOLEAN caps;	/* surname in caps? */
 BOOLEAN reg;	/* regular order? (not surname first) */
 INT len;	/* max name length */
 {
+	static unsigned char scratch[MAXNAMELEN+1];
 	if (!name || *name == 0) return NULL;
+	translate_string(tt, name, scratch, MAXNAMELEN+1);
+	name = scratch;
 	if (caps) name = upsurname(name);
 	name = trim_name(name, reg ? len: len-1);
 	if (reg) return trim(name_string(name), len);
 	return trim(name_surfirst(name), len);
 }
-/*================================================
- * name_string -- Removes slashes from GEDCOM name
- *==============================================*/
+/*===============================================
+ * name_string -- Remove slashes from GEDCOM name
+ *=============================================*/
 STRING name_string (name)
 STRING name;
 {
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING p = scratch;
 	ASSERT(strlen(name) <= MAXNAMELEN);
 	while (*name) {
@@ -732,7 +734,7 @@ STRING name;
 STRING name_surfirst (name)
 STRING name;
 {
-	static char scratch[MAXNAMELEN+1];
+	static unsigned char scratch[MAXNAMELEN+1];
 	STRING p = scratch;
 	ASSERT(strlen(name) <= MAXNAMELEN);
 	strcpy(p, getsurname(name));
@@ -750,24 +752,27 @@ STRING name;
 STRING **pkeys;
 {
 	STRING rec, str, p = name;
-	static char kbuf[MAXNAMELEN];
-	static char nbuf[MAXNAMELEN];
+	static unsigned char kbuf[MAXNAMELEN];
+	static unsigned char nbuf[MAXNAMELEN];
 	static STRING kaddr, naddr;
 	INT i = 0, c, len;
 	NODE indi;
 	while ((c = *p++) && chartype(c) == WHITE)
 		;
 	if (c == 0) return NULL;
-	if (c != 'I' && chartype(c) != DIGIT) return NULL;
+	if (c != 'I' && c != 'i' && chartype(c) != DIGIT) return NULL;
+	if (chartype(c) != DIGIT) c = *p++;
+	if (chartype(c) != DIGIT) return NULL;
 	kbuf[i++] = 'I';
-	if (c != 'I') kbuf[i++] = c;
+	kbuf[i++] = c;
 	while ((c = *p++) && chartype(c) == DIGIT)
 		kbuf[i++] = c;
 	if (c != 0) return NULL;
 	kbuf[i] = 0;
 	kaddr = kbuf;
 	*pkeys = &kaddr;
-	if (!(rec = getrecord(BTR, str2rkey(kbuf), &len))) return NULL;
+	if (!(rec = (STRING) getrecord(BTR, str2rkey(kbuf), &len)))
+		return NULL;
 	if (!(indi = string_to_node(rec))) {
 		stdfree(rec);
 		return NULL;
@@ -778,7 +783,7 @@ STRING **pkeys;
 	}
 	strcpy(nbuf, str);
 	naddr = nbuf;
-	stdfree(rec);	/* NEW 8/28/94 */
+	stdfree(rec);
 	return &naddr;
 }
 /*============================================
@@ -810,6 +815,5 @@ INT *psind;	/* index (rel 1) of surname in list */
 		set_list_element(list, i + 1, str);
 	}
 	*plen = i;
-	ASSERT(*psind);
 	return TRUE;
 }
