@@ -2,7 +2,7 @@
  * node.c -- Standard GEDCOM NODE operations
  * Copyright(c) 1992-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 04 Sep 93
- *   3.0.0 - 29 Aug 94
+ *   3.0.0 - 29 Aug 94    3.0.2 - 23 Dec 94
  *===========================================================*/
 
 #include "standard.h"
@@ -15,15 +15,15 @@ INT lineno = 0;
 
 static BOOLEAN buffer_to_line();
 
-STRING fileof = "The file is as positioned at EOF.";
-STRING reremp = "Line %d: This line is empty; EOF?";
-STRING rerlng = "Line %d: This line is too long.";
-STRING rernlv = "Line %d: This line has no level number.";
-STRING rerinc = "Line %d: This line is incomplete.";
-STRING rerbln = "Line %d: This line has a bad link.";
-STRING rernwt = "Line %d: This line needs white space before tag.";
-STRING rerilv = "Line %d: This line has an illegal level.";
-STRING rerwlv = "The record begins at wrong level.";
+STRING fileof = (STRING) "The file is as positioned at EOF.";
+STRING reremp = (STRING) "Line %d: This line is empty; EOF?";
+STRING rerlng = (STRING) "Line %d: This line is too long.";
+STRING rernlv = (STRING) "Line %d: This line has no level number.";
+STRING rerinc = (STRING) "Line %d: This line is incomplete.";
+STRING rerbln = (STRING) "Line %d: This line has a bad link.";
+STRING rernwt = (STRING) "Line %d: This line needs white space before tag.";
+STRING rerilv = (STRING) "Line %d: This line has an illegal level.";
+STRING rerwlv = (STRING) "The record begins at wrong level.";
 
 /*==============================
  * fixup -- Save non-tag strings
@@ -168,11 +168,11 @@ STRING p;
 INT *plev;
 STRING *pxref, *ptag, *pval, *pmsg;
 {
-	static char zero = 0;
+	static unsigned char zero = 0;
 	INT lev;
 	extern INT lineno;
 	STRING p0 = p;
-	static char scratch[MAXLINELEN+40];
+	static unsigned char scratch[MAXLINELEN+40];
 	*pmsg = 0;
 	*pxref = *pval = &zero;
 	if (!p || *p == 0) {
@@ -256,7 +256,7 @@ BOOLEAN *pemp;	/* set true if file is empty */
 {
 	FILE *fp;
 	NODE node;
-	static char scratch[100];
+	static unsigned char scratch[100];
 	*pmsg = NULL;
 	*pemp = FALSE;
 	if (!(fp = fopen(fname, "r"))) {
@@ -313,7 +313,7 @@ BOOLEAN *peof;	/* set true if file is at end of file */
 {
 	INT curlev, bcode, rc;
 	NODE root, node, curnode;
-	static char scratch[100];
+	static unsigned char scratch[100];
 	*pmsg = NULL;
 	*peof = FALSE;
 	if (ateof) {
@@ -466,7 +466,7 @@ TRANTABLE tt;	/* char map */
 NODE node;	/* node */
 BOOLEAN indent;	/* indent? */
 {
-	char out[MAXLINELEN+1];
+	unsigned char out[MAXLINELEN+1];
 	STRING p = out;
 	if (indent) {
 		INT i;
@@ -510,7 +510,7 @@ INT levl;	/* level */
 NODE node;	/* node */
 STRING p;	/* write string */
 {
-	char scratch[600];
+	unsigned char scratch[600];
 	STRING q = scratch;
 	sprintf(q, "%d ", levl);
 	q += strlen(q);
@@ -556,7 +556,7 @@ NODE node;	/* root */
 	INT len = tree_strlen(0, node) + 1;
 	STRING str;
 	if (len <= 0) return NULL;
-	str = stdalloc(len);
+	str = (STRING) stdalloc(len);
 	(void) swrite_nodes(0, node, str);
 	return str;
 }
@@ -584,7 +584,7 @@ INT levl;	/* level */
 NODE node;	/* node */
 {
 	INT len;
-	char scratch[10];
+	unsigned char scratch[10];
 	sprintf(scratch, "%d", levl);
 	len = strlen(scratch) + 1;
 	if (nxref(node)) len += strlen(nxref(node)) + 1;
@@ -592,26 +592,56 @@ NODE node;	/* node */
 	if (nval(node)) len += strlen(nval(node)) + 1;
 	return len + 1;
 }
-/*=========================================
- * fam_to_dbase -- Store family in database
- *=======================================*/
-fam_to_dbase (node)
-NODE node;
-{
-	STRING str;
-	if (!node || strcmp("FAM", ntag(node))) return;
-	str = node_to_string(node);
-	ASSERT(store_record(rmvat(nxref(node)), str, strlen(str)));
-	stdfree(str);
-}
 /*==========================================
  * indi_to_dbase -- Store person in database
  *========================================*/
 indi_to_dbase (node)
 NODE node;
+{
+	node_to_dbase(node, "INDI");
+}
+/*=========================================
+ * fam_to_dbase -- Store family in database
+ *=======================================*/
+fam_to_dbase (node)
+NODE node;
+{
+	node_to_dbase(node, "FAM");
+}
+/*=========================================
+ * even_to_dbase -- Store event in database
+ *=======================================*/
+even_to_dbase (node)
+NODE node;
+{
+	node_to_dbase(node, "EVEN");
+}
+/*==========================================
+ * sour_to_dbase -- Store source in database
+ *========================================*/
+sour_to_dbase (node)
+NODE node;
+{
+	node_to_dbase(node, "SOUR");
+}
+/*================================================
+ * othr_to_dbase -- Store other record in database
+ *==============================================*/
+othr_to_dbase (node)
+NODE node;
+{
+	node_to_dbase(node, NULL);
+}
+/*===============================================
+ * node_to_dbase -- Store GEDCOM tree in database
+ *=============================================*/
+node_to_dbase (node, tag)
+NODE node;
+STRING tag;
 {
 	STRING str;
-	if (!node || strcmp("INDI", ntag(node))) return;
+	ASSERT(node);
+	if (tag) { ASSERT(eqstr(tag, ntag(node))); }
 	str = node_to_string(node);
 	ASSERT(store_record(rmvat(nxref(node)), str, strlen(str)));
 	stdfree(str);
@@ -699,7 +729,7 @@ NODE indi;
 	prev = NULL;
 	chil = CHIL(fam);
 	while (chil) {
-		if (!strcmp(nxref(indi), nval(chil))) {
+		if (eqstr(nxref(indi), nval(chil))) {
 			if (!prev) return NULL;
 			return key_to_indi(rmvat(nval(prev)));
 		}
@@ -719,7 +749,7 @@ NODE indi;
 	if (!(fam = indi_to_famc(indi))) return NULL;
 	chil = CHIL(fam);
 	while (chil) {
-		if (!strcmp(nxref(indi), nval(chil))) {
+		if (eqstr(nxref(indi), nval(chil))) {
 			chil = nsibling(chil);
 			if (!chil) return NULL;
 			return key_to_indi(rmvat(nval(chil)));
@@ -731,29 +761,31 @@ NODE indi;
 /*======================================
  * indi_to_name -- Return name of person
  *====================================*/
-STRING indi_to_name (node, len)
+STRING indi_to_name (node, tt, len)
 NODE node;
+TRANTABLE tt;
 INT len;
 {
-	if (!node) return "NO NAME";
-	if (!(node = find_tag(nchild(node), "NAME"))) return "NO NAME";
-	return manip_name(nval(node), TRUE, TRUE, len);
+	if (!node) return (STRING) "NO NAME";
+	if (!(node = find_tag(nchild(node), "NAME"))) return (STRING) "NO NAME";
+	return manip_name(nval(node), tt, TRUE, TRUE, len);
 }
 /*==============================================
  * indi_to_event -- Convert event tree to string
  *============================================*/
-STRING indi_to_event (node, tag, head, len, shrt)
+STRING indi_to_event (node, tt, tag, head, len, shrt)
 NODE node;
+TRANTABLE tt;
 STRING tag, head;
 INT len;
 BOOLEAN shrt;
 {
-	static char scratch[200];
+	static unsigned char scratch[200];
 	STRING event;
 	INT n;
 	if (!node) return NULL;
 	if (!(node = find_tag(nchild(node), tag))) return NULL;
-	event = event_to_string(node, shrt);
+	event = event_to_string(node, tt, shrt);
 	if (!event) return NULL;
 	sprintf(scratch, "%s%s", head, event);
 	n = strlen(scratch);
@@ -767,11 +799,13 @@ BOOLEAN shrt;
 /*===========================================
  * event_to_string -- Convert event to string
  *=========================================*/
-STRING event_to_string (node, shrt)
+STRING event_to_string (node, tt, shrt)
 NODE node;
+TRANTABLE tt;
 BOOLEAN shrt;
 {
-	static char scratch[200];
+	static unsigned char scratch1[MAXLINELEN+1];
+	static unsigned char scratch2[MAXLINELEN+1];
 	STRING date, plac, p;
 	date = plac = NULL;
 	node = nchild(node);
@@ -786,7 +820,7 @@ BOOLEAN shrt;
 		plac = shorten_plac(plac);
 		if (!date && !plac) return NULL;
 	}
-	p = scratch;
+	p = scratch1;
 	if (date && !plac) {
 		strcpy(p, date);
 		p += strlen(p);
@@ -804,20 +838,23 @@ BOOLEAN shrt;
 		p += strlen(p);
 	}
 	*p = 0;
-	return scratch;
+	translate_string(tt, scratch1, scratch2, MAXLINELEN);
+	return scratch2;
 }
 /*=======================================
  * event_to_date -- Convert event to date
  *=====================================*/
-STRING event_to_date (node, shrt)
+STRING event_to_date (node, tt, shrt)
 NODE node;
+TRANTABLE tt;
 BOOLEAN shrt;
 {
+	static unsigned char scratch[MAXLINELEN+1];
 	if (!node) return NULL;
-	node = DATE(node);
-	if (!node) return NULL;
-	if (shrt) return shorten_date(nval(node));
-	return nval(node);
+	if (!(node = DATE(node))) return NULL;
+	translate_string(tt, nval(node), scratch, MAXLINELEN);
+	if (shrt) return shorten_date(scratch);
+	return scratch;
 }
 /*========================================
  * event_to_plac -- Convert event to place
@@ -838,7 +875,7 @@ BOOLEAN shrt;
 striptrail (p)
 STRING p;
 {
-	char *q	= p + strlen(p) - 1;
+	unsigned char *q = p + strlen(p) - 1;
 	while (iswhite(*q) && q >= p)
 		*q-- = 0;
 }
@@ -898,7 +935,7 @@ NODE node;
 STRING shorten_date (date)
 STRING date;
 {
-	static char buffer[3][MAXLINELEN+1];
+	static unsigned char buffer[3][MAXLINELEN+1];
 	static int dex = 0;
 	STRING p = date, q;
 	INT c, len;
@@ -932,7 +969,7 @@ STRING plac;
 {
 	STRING plac0 = plac, comma, val;
 	if (!plac) return NULL;
-	comma = strrchr(plac, ',');
+	comma = (STRING) strrchr(plac, ',');
 	if (comma) plac = comma + 1;
 	while (*plac++ == ' ')
 		;
@@ -990,16 +1027,18 @@ BOOLEAN kids, sibs;
 /*===============================================================
  * traverse_nodes -- Traverse nodes in tree while doing something
  *=============================================================*/
+INT tlineno;
 BOOLEAN traverse_nodes (node, func)
 NODE node;		/* root of tree to traverse */
 BOOLEAN (*func)();	/* function to call at each node */
 {
 	while (node) {
+		tlineno++;
+		if (!(*func)(node)) return FALSE;
 		if (nchild(node)) {
 			if (!traverse_nodes(nchild(node), func))
 				return FALSE;
 		}
-		if (!(*func)(node)) return FALSE;
 		node = nsibling(node);
 	}
 	return TRUE;
@@ -1015,3 +1054,25 @@ NODE indi;
 	FORSPOUSES(indi, spouse, fam, nsp) ENDSPOUSES
 	return nsp;
 }
+/*===================================================
+ * find_node -- Find node with specific tag and value
+ *=================================================*/
+NODE find_node (prnt, tag, val, plast)
+NODE prnt;	/* parent node */
+STRING tag;	/* tag, may be NULL */
+STRING val;	/* value, may be NULL */
+NODE *plast;	/* previous node, may be NULL */
+{
+	NODE last, node;
+
+	if (plast) *plast = NULL;
+	if (!prnt || (!tag && !val)) return NULL;
+	for (last = NULL, node = nchild(prnt); node;
+	     last = node, node = nsibling(node)) {
+		if (tag && nestr(tag, ntag(node))) continue;
+		if (val && nestr(val, nval(node))) continue;
+		if (plast) *plast = last;
+		return node;
+	}
+	return NULL;
+}
