@@ -1,8 +1,9 @@
 /*=============================================================
  * screen.c -- Curses user interface to LifeLines
- * Copyright(c) 1992-94 by T.T. Wetmore IV; all rights reserved
+ * Copyright(c) 1992-95 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 29 Sep 93
  *   2.3.6 - 01 Jan 94    3.0.0 - 06 Oct 94
+ *   3.0.2 - 25 Mar 95
  *===========================================================*/
 
 #include "standard.h"
@@ -25,11 +26,11 @@
 STRING get_answer();
 WINDOW *choose_win();
 extern BOOLEAN alldone;
-extern STRING version, betaversion, empstr, empstr71, btreepath;
+extern STRING version, betaversion, empstr, empstr71, readpath;
 extern STRING abverr, uoperr;
-STRING mtitle = "LifeLines %s - Genealogical Database and Report Generator";
-STRING cright = "Copyright(c) 1991 to 1994, by T. T. Wetmore IV";
-STRING plschs = "Please choose an operation:";
+STRING mtitle = (STRING) "LifeLines %s - Genealogical Database and Report Generator";
+STRING cright = (STRING) "Copyright(c) 1991 to 1995, by T. T. Wetmore IV";
+STRING plschs = (STRING) "Please choose an operation:";
 BOOLEAN stdout_vis = FALSE;
 
 INDISEQ indiseq_list_interact();
@@ -84,7 +85,7 @@ paint_main_screen()
 	wprintw(win, mtitle, version);
 #endif
 	mvwaddstr(win, 2, 4, cright);
-	mvwprintw(win, 3, 4, "Current Database - %s", btreepath);
+	mvwprintw(win, 3, 4, "Current Database - %s", readpath);
 	if (readonly)
 		wprintw(win, " (read only)");
 	row = 5;
@@ -276,6 +277,31 @@ paint_list_screen ()
 	mvwaddstr(win, row++, col, "x  Swap mark/current");
 	mvwaddstr(win, row++, col, "q  Return to main menu");
 }
+/*===========================================
+ * paint_aux_screen -- Paint auxillary screen
+ *=========================================*/
+paint_aux_screen ()
+{
+	WINDOW *win = main_win;
+	INT row, col;
+	werase(win);
+	BOX(win, 0, 0);
+	show_horz_line(win, 16, 0, COLSREQ);
+	show_horz_line(win, LINESREQ-3, 0, COLSREQ);
+	mvwaddstr(win, 17, 2, plschs);
+	row = 18; col = 3;
+	mvwaddstr(win, row++, col, "e  Edit the record");
+	mvwaddstr(win, row++, col, "i  Browse to record");
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	row = 18; col = 3 + BAND;
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	row = 18; col = 3 + 2*BAND;
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	mvwaddstr(win, row++, col, "x  Not implemented");
+	mvwaddstr(win, row++, col, "q  Return to main menu");
+}
 /*==========================================
  * create_windows -- Create and init windows
  *========================================*/
@@ -311,7 +337,7 @@ create_windows ()
 	BOX(debug_box_win, 0, 0);
 }
 /*=====================================
- * init_all_windows -- Init all WINDOWs
+ * init_all_windows -- Init all windows
  *===================================*/
 init_all_windows ()
 {
@@ -346,7 +372,7 @@ init_all_windows ()
 	mvwaddstr(win, row++, 4, "x  Internal to GEDCOM mapping");
 	mvwaddstr(win, row++, 4, "d  Internal to Display mapping");
 	mvwaddstr(win, row++, 4, "r  Internal to Report mapping");
-	mvwaddstr(win, row++, 4, "q  Quit - return to the previous menu");
+	mvwaddstr(win, row++, 4, "q  Return to the main menu");
 
 	win = utils_menu_win;
 	row = 1;
@@ -363,7 +389,7 @@ init_all_windows ()
 
 	win = extra_menu_win;
 	row = 1;
-	mvwaddstr(win, row++, 2, "What utility do you want to perform?");
+	mvwaddstr(win, row++, 2, "What activity do you want to perform?");
 	mvwaddstr(win, row++, 4, "1  Add a source record to the database");
 	mvwaddstr(win, row++, 4, "2  Edit source record from the database");
 	mvwaddstr(win, row++, 4, "3  Add an event record to the database");
@@ -410,7 +436,7 @@ main_menu ()
 	case 'b': browse(NULL); break;
 	case 'a': add_menu(); break;
 	case 'd': del_menu(); break;
-	case 'r': interp_report(); break;
+	case 'r': interp_main(); break;
 	case 't': trans_menu(); break;
 	case 'u': utils_menu(); break;
 	case 'x': extra_menu(); break;
@@ -424,9 +450,9 @@ indi_browse (indi)
 NODE indi;
 {
 	if (cur_screen != ONE_PER_SCREEN) paint_one_per_screen();
-	show_person(indi, main_win, 1, 11);
+	show_person(indi, 1, 11);
 	display_screen(ONE_PER_SCREEN);
-	return interact(main_win, "efmscoygubhirdpnaxtzq");
+	return interact(main_win, "efmscoygubhirdpnaxtzqA");
 }
 /*=======================================
  * fam_browse -- Handle fam_browse screen
@@ -435,7 +461,7 @@ fam_browse (fam)
 NODE fam;
 {
 	if (cur_screen != ONE_FAM_SCREEN) paint_one_fam_screen();
-	show_long_family(fam, main_win, 1, 13);
+	show_long_family(fam, 1, 13);
 	display_screen(ONE_FAM_SCREEN);
 	return interact(main_win, "efmcnsardxtbzq");
 }
@@ -446,8 +472,8 @@ tandem_browse (indi1, indi2)
 NODE indi1, indi2;
 {
 	if (cur_screen != TWO_PER_SCREEN) paint_two_per_screen();
-	show_person(indi1, main_win, 1, 6);
-	show_person(indi2, main_win, 8, 6);
+	show_person(indi1, 1, 6);
+	show_person(indi2, 8, 6);
 	display_screen(TWO_PER_SCREEN);
 	return interact(main_win, "etfmscbdajxq");
 }
@@ -458,8 +484,8 @@ twofam_browse (fam1, fam2)
 NODE fam1, fam2;
 {
 	if (cur_screen != TWO_FAM_SCREEN) paint_two_fam_screen();
-	show_short_family(fam1, main_win, 1, 6);
-	show_short_family(fam2, main_win, 8, 6);
+	show_short_family(fam1, 1, 6);
+	show_short_family(fam2, 8, 6);
 	display_screen(TWO_FAM_SCREEN);
 	return interact(main_win, "etbfmxjq");
 }
@@ -474,6 +500,17 @@ NODE indi;
 	display_screen(PED_SCREEN);
 	return interact(main_win, "eifmscgbq");
 }
+/*=======================================
+ * aux_browse -- Handle aux_browse screen
+ *=====================================*/
+aux_browse (node)
+NODE node;
+{
+	if (cur_screen != AUX_SCREEN) paint_aux_screen();
+	show_aux_display(node, 1, 11);
+	display_screen(AUX_SCREEN);
+	return interact(main_win, "eq");
+}
 /*=========================================
  * list_browse -- Handle list_browse screen
  *=======================================*/
@@ -500,7 +537,7 @@ STRING ttl, prmpt;
 	mvwaddstr(win, 2, 1, prmpt);
 	wrefresh(win);
 	rv = get_answer(win, prmpt);
-	if (!rv) return "";
+	if (!rv) return (STRING) "";
 	p = rv;
 	while (chartype(*p) == WHITE)
 		p++;
@@ -642,7 +679,7 @@ del_menu ()
 	touchwin(main_win);
 	wrefresh(main_win);
 	switch (code) {
-	case 'c': remove_child(NULL, FALSE); break;
+	case 'c': remove_child(NULL, NULL, FALSE); break;
 	case 's': remove_spouse(NULL, NULL, FALSE); break;
 	case 'i': delete_indi(NULL, TRUE); break;
 	case 'q': break;
@@ -727,6 +764,9 @@ STRING str;
 	while (TRUE) {
 		crmode();
 		c = wgetch(win);
+#if 1
+		if(c == EOF) c = 'q';
+#endif
 		nocrmode();
 		now_showing = FALSE;
 		place_std_msg();
@@ -742,7 +782,7 @@ STRING get_answer (win, prmpt)
 WINDOW *win;
 STRING prmpt;
 {
-	static char lcl[100];
+	static unsigned char lcl[100];
 #ifndef BSD
 	echo();
 	mvwgetstr(win, 2, strlen(prmpt) + 2, lcl);
@@ -753,7 +793,7 @@ STRING prmpt;
 	return lcl;
 }
 /*===========================================================
- * win_list_init -- Create list of WINDOWs of increasing size
+ * win_list_init -- Create list of windows of increasing size
  *=========================================================*/
 static WINDOW *list_wins[VIEWABLE];
 win_list_init ()
@@ -901,7 +941,7 @@ INT len, top, cur;
  * list_interact -- Interact with user over list
  *============================================*/
 INT list_interact(win, ttl, len, strings)
-WINDOW *win;	/* interaction WINDOW */
+WINDOW *win;	/* interaction window */
 STRING ttl;	/* title */
 INT len;	/* list length */
 STRING *strings;/* string list */
@@ -972,21 +1012,23 @@ STRING message_string ()
 {
 	switch (cur_screen) {
 	case MAIN_SCREEN:
-		return "LifeLines -- Main Menu";
+		return (STRING) "LifeLines -- Main Menu";
 	case ONE_PER_SCREEN:
-		return "LifeLines -- Person Browse Screen";
+		return (STRING) "LifeLines -- Person Browse Screen";
 	case ONE_FAM_SCREEN:
-		return "LifeLines -- Family Browse Screen";
+		return (STRING) "LifeLines -- Family Browse Screen";
 	case PED_SCREEN:
-		return "LifeLines -- Pedigree Browse Screen";
+		return (STRING) "LifeLines -- Pedigree Browse Screen";
+	case AUX_SCREEN:
+		return (STRING) "LifeLines -- Auxilliary Browse Screen";
 	case TWO_PER_SCREEN:
-		return "LifeLines -- Tandem Browse Screen";
+		return (STRING) "LifeLines -- Tandem Browse Screen";
 	case TWO_FAM_SCREEN:
-		return "LifeLines -- Two Family Browse Screen";
+		return (STRING) "LifeLines -- Two Family Browse Screen";
 	case LIST_SCREEN:
-		return "LifeLines -- List Browse Screen";
+		return (STRING) "LifeLines -- List Browse Screen";
 	default:
-		return "";
+		return (STRING) "";
 	}
 }
 /*=================================================
@@ -1085,6 +1127,7 @@ place_cursor ()
 	case ONE_PER_SCREEN: row = LINESREQ-11; break;
 	case ONE_FAM_SCREEN: row = LINESREQ-9;  break;
 	case PED_SCREEN:     row = 17;       break;
+	case AUX_SCREEN:     row = 17;       break;
 	case TWO_PER_SCREEN: row = 15;       break;
 	case TWO_FAM_SCREEN: row = 15;       break;
 	case LIST_SCREEN:    row = 8; col = 75; break;
