@@ -2,7 +2,7 @@
  * screen.h -- Header file for curses-based screen I/O
  * Copyright (c) 1991-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93    2.3.5 - 20 Aug 93
- *   3.0.0 - 26 Jul 94
+ *   3.0.0 - 26 Jul 94    3.0.2 - 16 Dec 94
  *============================================================*/
 
 #undef TRUE
@@ -22,6 +22,7 @@
 #define TWO_FAM_SCREEN 5
 #define PED_SCREEN     6
 #define LIST_SCREEN    7
+#define AUX_SCREEN     8
 
 #ifndef ACS_TTEE
 #	define ACS_TTEE '+'
