@@ -2,7 +2,7 @@
  * valgdcom.c -- Validate GEDCOM file
  * Copyright(c) 1993-94 by T.T. Wetmore IV; all rights reserved
  *   2.3.4 - 24 Jun 93     2.3.5 - 02 Sep 93
- *   3.0.0 - 06 Jul 94
+ *   3.0.0 - 06 Jul 94     3.0.2 - 18 Feb 95
  *===========================================================*/
 
 #include "standard.h"
@@ -30,35 +30,37 @@ static FILE *flog;
 
 ELMNT *index_data = NULL;
 
-STRING misixr = "Line %d: The person defined here has no key.";
-STRING misfxr = "Line %d: The family defined here has no key.";
-STRING misexr = "Line %d: The event defined here has no key.";
-STRING missxr = "Line %d: The source defined here has no key.";
-STRING misrxr = "Line %d: The record defined here has no key.";
-STRING mulper = "Lines %d and %d: Person %s is multiply defined.";
-STRING mulfam = "Lines %d and %d: Family %s is multiply defined.";
-STRING mulsrc = "Lines %d and %d: Source %s is multiply defined.";
-STRING mulevn = "Lines %d and %d: Event %s is multiply defined.";
-STRING muloth = "Lines %d and %d: Record %s is multiply defined.";
-STRING matper = "Line %d: Person %s has an incorrect key.";
-STRING matfam = "Line %d: Family %s has an incorrect key.";
-STRING matsrc = "Line %d: Source %s has an incorrect key.";
-STRING matevn = "Line %d: Event %s has an incorrect key.";
-STRING matoth = "Line %d: Record %s has an incorrect key.";
-STRING misval = "Line %d: This %s line is missing a value field.";
-STRING undper = "Person %s is referred to but not defined.";
-STRING undfam = "Family %s is referred to but not defined.";
-STRING undrec = "Record %s is referred to but not defined.";
-STRING undsrc = "Source %s is referred to but not defined.";
-STRING undevn = "Event %s is referred to but not defined.";
-STRING badlev = "Line %d: This line has a level number that is too large.";
-STRING noname = "Line %d: Person defined here has no name.";
-STRING mulfth = "Line %d: Person %s has multiple father links.";
-STRING mulmth = "Line %d: Person %s has multiple mother links.";
-STRING mulhsb = "Line %d: Family %s has multiple husband links.";
-STRING mulwif = "Line %d: Person %s has multiple wife links.";
+#define SS (STRING)
+
+STRING misixr = SS "Line %d: The person defined here has no key.";
+STRING misfxr = SS "Line %d: The family defined here has no key.";
+STRING misexr = SS "Line %d: The event defined here has no key.";
+STRING missxr = SS "Line %d: The source defined here has no key.";
+STRING misrxr = SS "Line %d: The record defined here has no key.";
+STRING mulper = SS "Lines %d and %d: Person %s is multiply defined.";
+STRING mulfam = SS "Lines %d and %d: Family %s is multiply defined.";
+STRING mulsrc = SS "Lines %d and %d: Source %s is multiply defined.";
+STRING mulevn = SS "Lines %d and %d: Event %s is multiply defined.";
+STRING muloth = SS "Lines %d and %d: Record %s is multiply defined.";
+STRING matper = SS "Line %d: Person %s has an incorrect key.";
+STRING matfam = SS "Line %d: Family %s has an incorrect key.";
+STRING matsrc = SS "Line %d: Source %s has an incorrect key.";
+STRING matevn = SS "Line %d: Event %s has an incorrect key.";
+STRING matoth = SS "Line %d: Record %s has an incorrect key.";
+STRING misval = SS "Line %d: This %s line is missing a value field.";
+STRING undper = SS "Person %s is referred to but not defined.";
+STRING undfam = SS "Family %s is referred to but not defined.";
+STRING undrec = SS "Record %s is referred to but not defined.";
+STRING undsrc = SS "Source %s is referred to but not defined.";
+STRING undevn = SS "Event %s is referred to but not defined.";
+STRING badlev = SS "Line %d: This line has a level number that is too large.";
+STRING noname = SS "Line %d: Person defined here has no name.";
+STRING mulfth = SS "Line %d: Person %s has multiple father links.";
+STRING mulmth = SS "Line %d: Person %s has multiple mother links.";
+STRING mulhsb = SS "Line %d: Family %s has multiple husband links.";
+STRING mulwif = SS "Line %d: Person %s has multiple wife links.";
 #if 0
-STRING noxref = "Line %d: This record has no cross reference value.";
+STRING noxref = SS "Line %d: This record has no cross reference value.";
 #endif
 
 static INT add_even_defn();
@@ -86,6 +88,7 @@ FILE *fp;
 {
 	INT lev, rc, curlev = 0;
 	INT nindi, nfam, nsour, neven, nothr;
+	extern INT lineno;
 	ELMNT el;
 	TRANTABLE tt = tran_tables[MGDIN];
 	STRING xref, tag, val, msg;
@@ -116,8 +119,8 @@ FILE *fp;
 			if (rc == ERROR)
 				handle_err(msg);
 			else
-				handle_err(badlev);
-			handle_value(val, noline);
+				handle_err(badlev, lineno);
+			handle_value(val, lineno);
 			curlev = lev;
 			rc = file_to_line(fp, tt, &lev, &xref, &tag, &val,
 			    &msg);
@@ -126,7 +129,7 @@ FILE *fp;
 			continue;
 		}
 		if (lev > 1) {
-			handle_value(val, noline);
+			handle_value(val, lineno);
 			curlev = lev;
 			rc = file_to_line(fp, tt, &lev, &xref, &tag, &val,
 			    &msg);
@@ -138,7 +141,7 @@ FILE *fp;
 			char str[10];
 			if (rec_type == INDI_REC && !named)
 				handle_err(noname, defline);
-			defline = noline;
+			defline = lineno;
 			if (eqstr("HEAD", tag) || eqstr("TRLR", tag))
 				rec_type = IGNR_REC;
 			else if (eqstr("INDI", tag)) {
@@ -146,35 +149,35 @@ FILE *fp;
 				wfield(1, 1, str);
 				rec_type = INDI_REC;
 				named = FALSE;
-				person = add_indi_defn(xref, noline, &el);
+				person = add_indi_defn(xref, lineno, &el);
 			} else if (eqstr("FAM", tag)) {
 				sprintf(str, "%6d", ++nfam);
 				wfield(2, 1, str);
 				rec_type = FAM_REC;
-				family = add_fam_defn(xref, noline);
+				family = add_fam_defn(xref, lineno);
 			} else if (eqstr("EVEN", tag)) {
 				sprintf(str, "%6d", ++neven);
 				wfield(3, 1, str);
 				rec_type = EVEN_REC;
-				event = add_even_defn(xref, noline);
+				event = add_even_defn(xref, lineno);
 			} else if (eqstr("SOUR", tag)) {
 				sprintf(str, "%6d", ++nsour);
 				wfield(4, 1, str);
 				rec_type = SOUR_REC;
-				source = add_sour_defn(xref, noline);
+				source = add_sour_defn(xref, lineno);
 			} else {
 				sprintf(str, "%6d", ++nothr);
 				wfield(5, 1, str);
 				rec_type = OTHR_REC;
-				other = add_othr_defn(xref, noline);
+				other = add_othr_defn(xref, lineno);
 			}
 		} else {
 			if (rec_type == INDI_REC)
-				handle_indi_lev1(tag, val, noline);
+				handle_indi_lev1(tag, val, lineno);
 			else if (rec_type == FAM_REC)
-				handle_fam_lev1(tag, val, noline);
+				handle_fam_lev1(tag, val, lineno);
 			else
-				handle_value(val, noline);
+				handle_value(val, lineno);
 		}
 		curlev = lev;
 		rc = file_to_line(fp, tt, &lev, &xref, &tag, &val, &msg);
@@ -453,14 +456,15 @@ INT line;
 {
 	INT dex;
 	ELMNT fam, pers;
-	ASSERT(family != -1);
-	fam = index_data[family];
+/*wprintf("handle_fam_lev1: %s, %s, %d\n",tag,val,line);/*DEBUG*/
+/*wprintf("handle_fam_lev1: family == %d\n", family);/*DEBUG*/
+	fam = (family != -1) ? index_data[family] : NULL;
 	if (eqstr(tag, "HUSB")) {
 		if (!pointer_value(val)) {
 			handle_err(misval, line, "HUSB");
 			return;
 		}
-		Male(fam) += 1;
+		if (fam) Male(fam) += 1;
 		if ((dex = add_indi_defn(rmvat(val), 0, &pers)) != -1)
 			Sex(pers) |= BE_MALE;
 	} else if (eqstr(tag, "WIFE")) {
@@ -468,7 +472,7 @@ INT line;
 			handle_err(misval, line, "WIFE");
 			return;
 		}
-		Fmle(fam) += 1;
+		if (fam) Fmle(fam) += 1;
 		if ((dex = add_indi_defn(rmvat(val), 0, &pers)) != -1)
 			Sex(pers) |= BE_FEMALE;
 	} else if (eqstr(tag, "CHIL")) {
